import requests, string
from requests.utils import requote_uri
from bs4 import BeautifulSoup, NavigableString, Tag
from skmds.classes import *

# search for page

def search(municipality):
    """ Search the mds for a given municipality
    :param: municipality: the name of a municipality
    :return: str: the uri of municipality if it exists; otherwise None
    """
    muni = municipality.replace("'", "''")
    url = requote_uri(f'http://www.mds.gov.sk.ca/apps/Pub/MDS/pubsrchres.aspx?muni={muni}&ReturnURL=welcome.aspx')
    html = requests.get(url).text
    soup = BeautifulSoup(html, 'html.parser')
    list = soup.find('table', attrs={'id': 'tblMunis'})
    results = list.findChildren("a" , recursive=True)
    if len(results) <= 0:
        return None
    return results[0]['href']
    
    
# Parsing results page content and retreiving data:

def _parse_details(soup):
    """ Parse the municipality info, mail address, and office address from mds page
    :param: soup: the bs4 soup of the page
    :return: dict: the municipality details
    """
    municipality = {}
    municipality['mail_address'] = {}
    municipality['office_address'] = {}
    
    # loop through html elements in depth first order
    for el in soup.find('fieldset').recursiveChildGenerator():
        if isinstance(el, NavigableString):
            continue
        if isinstance(el, Tag) and el.has_attr('id'):
            tag_id = el.get('id').lower()[3:]
            # parse name and category
            if tag_id == 'muni':  
                words = el.text.split(' of ')
                res = string.capwords(words[-1].lower()), words[0].lower()
                municipality['name'] = res[0]
                municipality['category'] = res[1]
            # parse assessment and population
            elif tag_id == 'assessment' or tag_id == 'population':
                if el.text == '':
                    municipality[tag_id] = None
                else:
                    municipality[tag_id] = int(el.text.replace(",", '').strip()) 
            # parse rm number
            elif tag_id == 'rmnumber':  
                try:
                    municipality['rm_number'] = int(el.text)
                except:
                    municipality['rm_number'] = None
            # parse municipality contact info
            elif tag_id == 'phone' or tag_id == 'fax' or tag_id == 'email' or tag_id == 'website':  
                if el.text == '':
                    municipality[tag_id] = None
                else:
                    municipality[tag_id] = el.text
            # parse municipality mail address
            elif tag_id[:4] == 'mail':  
                municipality['mail_address'][tag_id[4:]] = el.text
            # parse office address
            elif tag_id[:6] == 'office':  
                tag = tag_id[6:]
                if tag != 'info':
                    municipality['office_address'][tag] = el.text
    
    municipality = Municipality(municipality)
    return municipality


def _parse_personnel(soup):
    """ Parse the personnel data from the table on the mds page
    :param: soup: the bs4 soup of the page
    :return: list: list of all personnel
    """
    personnel = []
    personnel_table = soup.find('table', attrs={'id': 'tblPersonnel'})
    rows = personnel_table.find_all('tr')
    for row in rows:
        cols = row.find_all('td')
        cols = [el.text.strip() for el in cols]
        person = Personnel(cols[0][:-1], cols[1], cols[2])
        personnel.append(person)
    return personnel


def get_data(uri):
    """ Retrieve the data of a municipality from the mds page.
    :param: uri: the uri of the mds page of the municipality
    :return: dict: municipal data from the mds
    """
    url = 'http://www.mds.gov.sk.ca/apps/Pub/MDS/' + uri
    html = requests.get(url).text
    soup = BeautifulSoup(html, 'html.parser')

    municipality = _parse_details(soup)
    municipality.personnel = _parse_personnel(soup)
    return municipality


def query(municipality):
    """ Query the mds for the data of a given municipality.
    :param: municipality: a string containing the name of the municipality to be queried
    :return: dict: the mds data if the municipality exists; otherwise None
    """
    uri = search(municipality)
    if uri is None:
        return None
    return get_data(uri)
